#include <qpainter.h>
#include <qbitmap.h>
#include <qdatetime.h>
#include <qdrawutil.h>
#include <qlcdnumber.h>
#include <qtimer.h>

#include <kdatepicker.h>
#include <kglobal.h>
#include <kwin.h>
#include <kiconloader.h>
#include <kpixmap.h>
#include <kpixmapeffect.h>
#include <klocale.h>
#include <kstandarddirs.h>
#include <kpopupmenu.h>

#include <taskmanager.h>

#include "kaspopup.h"
#include "kastasker.h"

#include "kasclockitem.h"
#include "kasclockitem.moc"

class LCD : public QLCDNumber
{
public:
    LCD( QWidget *parent, const char *name=0 )
	: QLCDNumber(parent,name) {}
    ~LCD() {}

    void draw( QPainter *p ) { drawContents(p); }
};

KasClockItem::KasClockItem( KasBar *parent )
    : KasItem( parent )
{
    QTimer *t = new QTimer( this );
    connect( t, SIGNAL( timeout() ), SLOT( updateTime() ) );
    t->start( 500 );

    lcd = new LCD( parent );
    lcd->hide();

    lcd->setBackgroundMode( NoBackground );
    lcd->setFrameStyle( QFrame::NoFrame );
    lcd->setSegmentStyle( QLCDNumber::Flat );
    lcd->setNumDigits( 5 );
    lcd->setAutoMask( true );
    updateTime();

    KasPopup *pop = new KasPopup( this );
    setPopup( pop );
    setCustomPopup( pop );

    (void) new KDatePicker( pop );
    pop->adjustSize();

    connect( this, SIGNAL(leftButtonClicked(QMouseEvent *)), SLOT(togglePopup()) );
    connect( this, SIGNAL(rightButtonClicked(QMouseEvent *)), SLOT(showMenuAt(QMouseEvent *) ) );
}

KasClockItem::~KasClockItem()
{
//    lcd->deleteLater();
}

void KasClockItem::updateTime()
{
    setText( QDate::currentDate().toString("d MMM") );
    lcd->display( QTime::currentTime().toString("hh:mm") );

    update();
}

void KasClockItem::paint( QPainter *p )
{
    KasItem::paint( p );

    lcd->setGeometry( QRect( 1, 1, extent()-2, extent()-18 ) );

    int h = lcd->sizeHint().height();
    int y = ((extent()-18)-h)/2+18;

    p->save();

    p->translate( 3, y-h+11 );
    lcd->setPaletteForegroundColor( kasbar()->colorGroup().mid() );
    lcd->draw( p );

    p->restore();
    p->save();

    p->translate( 1, y-h+9 );
    lcd->setPaletteForegroundColor( kasbar()->resources()->activePenColor() );
    lcd->draw( p );

    p->restore();
}

void KasClockItem::showMenuAt( QMouseEvent *ev )
{
    hidePopup();
    showMenuAt( ev->globalPos() );
}

void KasClockItem::showMenuAt( QPoint p )
{
    mouseLeave();
    kasbar()->updateMouseOver();

    KasTasker *bar = dynamic_cast<KasTasker *> (KasItem::kasbar());
    if ( !bar )
	return;

    KPopupMenu *menu = bar->contextMenu();
    menu->exec( p );
}
